# Little library for parsing articles
# chmod +x lib/run_parser.rb
# run from Rails root bundle exec lib/run_parser.rb
require 'pry'
require 'open-uri'
require 'yaml'
require 'mechanize'
require 'benchmark'

BASE_PATH = File.expand_path("fly_parser/*.rb", File.dirname(__FILE__))
LOGO_PATH = File.expand_path("fly_parser/logo.txt", File.dirname(__FILE__))

Pry.config.print = proc { |output, value| output.puts value.ai }

# Require all of the Ruby files in the given directory.
#
# path - The String relative path from here to the directory.
#
# Returns nothing.
def require_all(path)
  glob = File.expand_path(File.join(File.dirname(__FILE__), path, '**', '*.rb'))
  Dir[glob].each do |f|
    require f
  end
end

Dir[BASE_PATH].each do |base_file|
  require base_file
end

# Require whole rails app , not only ActiveRecord
# because there is a lot of AR dependecies ( carrierwave, minimagick, state_machine etc )

Dir.chdir RAILS_ROOT
require RAILS_BOOT_PATH
require RAILS_CONFIG_PATH
require_all 'fly_parser/sources'

module Parser
  class << self
    include Enable if defined? Enable
    # Get HTTP Source
    def http(url)
      Nokogiri::HTML(open(url))
    end

    def connect(url)
      agent = Mechanize.new
      agent.pluggable_parser.default = Mechanize::Page
      agent.get(url)
    end

    def save(articles, options)
      articles.each do |article|
        item = Article.new(title: article[:title], content: article[:content])
        item.categories = [Category.find(options[:category_id])]
        item.remote_image_url = article[:poster_image]
        next unless item.valid?
        item.save

        item.tags << tags(article[:tags]) if article[:tags]

      end
    end

    def tags(tags)
      tags.map { |tag_name| Tag.find_or_create_by!(title: tag_name) }
    end

    def start
      puts Time.now

      source = find_source

      init_parser(source)
      parse_and_save(source["items"])
    end
    # Threads are more slowly in this case, so don't use it
    # because we need delay(some sources ban you for a lot of requests concurrently)
    def parse_and_save(items)
      items.each do |item|
        ap "Parsing #{item['type']}"
        result = parse_item(item)
        save_item(item, result)
      end
    end

    def save_item(item, result)
      category = Category.find_or_create_by!(name: JSON.generate(en: item["category"]))
      ap "and save to #{category.localized_name} category"
      Parser.save result, {category_id: category.id}
    end

    def parse_item(item)
      item["parser"].parse_all
    end

    def config
      YAML.load_file(CONFIG_PATH)
    end

    def find_source
      config["sources"].find {|source| source["enabled"] }
    end

    def logo
      File.read(LOGO_PATH)
    end

    def init_parser(source)
      source_type = source["source"].gsub('-', '_')
      prefix = "enable_"
      send(prefix + source_type, source)
    end

  end

end
